import unittest

from assignments import Assignments
from concatenation import Concatenation
from expression import Expression
from file_path import FilePath
from file_range import FileRange, parse
from literal import Literal
from name import Name
from variable import Variable


class TestAssignments(unittest.TestCase):
    variable: Variable
    expression: Expression
    assignments: Assignments

    def setUp(self) -> None:
        name: Name = Name("name")
        file_path: FilePath = FilePath("don't care")
        file_range: FileRange = parse("1:1-2:2")
        self.variable = Variable(name, file_path, file_range)
        self.expression = Literal("don't care")
        self.assignments = Assignments()

    def test_equality_empty(self):
        self.assertEqual(self.assignments, self.assignments)

    def test_equality_singleton(self):
        self.assignments.add(self.variable, self.expression)
        self.assertEqual(self.assignments, self.assignments)

    def test_inequality_empty_and_singleton(self):
        self.assignments.add(self.variable, self.expression)
        self.assertNotEqual(self.assignments, Assignments())

    def test_substitute_identity(self):
        self.assignments.add(self.variable, self.variable)
        actual = self.assignments.substitute_in_expression(self.variable)
        self.assertEqual(self.variable, actual)

    def test_substitute_variable_by_expression(self):
        self.assignments.add(self.variable, self.expression)
        actual = self.assignments.substitute_in_expression(self.variable)
        self.assertEqual(self.expression, actual)

    def test_substitute_in_concatenation(self):
        self.assignments.add(self.variable, self.expression)
        concatenation_list = list()
        expected_list = list()
        self.assertEqual(Concatenation(expected_list),
                         self.assignments.substitute_in_sequence(Concatenation(concatenation_list)))
        for i in range(1, 11):
            with self.subTest(str(i)):
                concatenation_list.append(self.variable)
                expected_list.append(self.expression)
                actual = self.assignments.substitute_in_sequence(Concatenation(concatenation_list))
                self.assertEqual(Concatenation(expected_list), actual)

    def test_substitute_in_assignment(self):
        another_name: Name = Name("another name")
        file_path: FilePath = FilePath("don't care")
        file_range: FileRange = parse("3:3-4:4")
        another_variable = Variable(another_name, file_path, file_range)
        self.assignments.add(self.variable, another_variable)
        self.assignments.add(another_variable, self.variable)

        expected = Assignments()
        expected.add(self.variable, self.variable)
        expected.add(another_variable, another_variable)
        self.assertEqual(expected, self.assignments.substitute_in_assignments(self.assignments))


if __name__ == '__main__':
    unittest.main()
