import unittest
from concatenation import Concatenation
from expression import Expression
from file_path import FilePath
from file_range import parse
from literal import Literal
from name import Name
from parameter import Parameter


class TestConcatenation(unittest.TestCase):
    def test_concatenation_is_expression(self):
        concatenation = Concatenation(tuple())
        self.assertIsInstance(concatenation, Expression)

    def test_empty_concatenation_is_closed(self):
        concatenation = Concatenation(tuple())
        self.assertTrue(concatenation.is_closed())

    def test_closed_only(self):
        list_of_closed_expressions = list()
        for i in range(1, 11):
            with self.subTest(str(i)):
                closed_expression = Concatenation(tuple())
                assert closed_expression.is_closed(), "Expectation of closed expression is violated"
                list_of_closed_expressions.append(closed_expression)
                concatenation = Concatenation(list_of_closed_expressions)
                self.assertEqual(i, len(concatenation))
                self.assertTrue(concatenation.is_closed())

    def test_literals_only(self):
        list_of_literals = list()
        for i in range(1, 11):
            with self.subTest(str(i)):
                literal_expression = Literal("don't care")
                assert literal_expression.is_closed(), \
                    "Expectation of literal expression is violated"
                list_of_literals.append(literal_expression)
                concatenation = Concatenation(list_of_literals)
                self.assertEqual(i, len(concatenation))
                self.assertTrue(concatenation.is_literal())

    def test_non_literal(self):
        name: Name = Name("don't care")
        file_path = FilePath("a file path")
        file_range = parse("1:1-2:2")
        non_literal_expression = Parameter(name, file_path, file_range)
        assert not isinstance(non_literal_expression, Literal), \
            "Expectation of non literal expression is violated"
        concatenation = Concatenation([non_literal_expression])
        self.assertFalse(concatenation.is_literal())

    def test_open(self):
        name: Name = Name("don't care")
        file_path = FilePath("a file path")
        file_range = parse("1:1-2:2")
        open_expression = Parameter(name, file_path, file_range)
        assert not open_expression.is_closed(), "Expectation of open expression is violated"
        concatenation = Concatenation([open_expression])
        self.assertFalse(concatenation.is_closed())

    def test_equality(self):
        """
        Equality by value
        """
        list_of_expressions = list()
        for i in range(1, 11):
            with self.subTest(str(i)):
                literal_expression = Literal(str(i))
                list_of_expressions.append(literal_expression)
                tuple_expression_left = Concatenation(list_of_expressions)
                tuple_expression_right = Concatenation(list_of_expressions)
                self.assertEqual(tuple_expression_left, tuple_expression_right)


if __name__ == '__main__':
    unittest.main()
