import unittest

from file import File
from file_path import FilePath
from file_range import parse
from parameter_declarations import ParameterDeclarations
from subprogram_declaration import SubprogramDeclaration

a_file_path = FilePath("a file path")
another_file_path = FilePath("another file path")
assert a_file_path != another_file_path

a_file_range = parse("1:1-2:2")
no_parameters = ParameterDeclarations()


class TestFile(unittest.TestCase):
    def test_file_has_initially_no_subprogram_declarations(self):
        file = File(a_file_path)
        self.assertEqual(0, len(file.subprogram_declarations))

    def test_cannot_add_subprogram_declaration_from_another_file(self):
        file = File(a_file_path)
        subprogram_declaration = SubprogramDeclaration("don't care",
                                                       another_file_path,
                                                       a_file_range,
                                                       no_parameters)
        with self.assertRaises(Exception):
            file.add_subprogram_declaration(subprogram_declaration)

    def test_add_subprogram_declaration_makes_length_one(self):
        file = File(a_file_path)
        subprogram_declaration = SubprogramDeclaration("don't care",
                                                       a_file_path,
                                                       a_file_range,
                                                       no_parameters)
        file.add_subprogram_declaration(subprogram_declaration)
        self.assertEqual(1, len(file.subprogram_declarations))

    def test_added_subprogram_declaration_can_be_retrieved_by_file_range(self):
        file = File(a_file_path)
        subprogram_declaration = SubprogramDeclaration("don't care",
                                                       a_file_path,
                                                       a_file_range,
                                                       no_parameters)
        file.add_subprogram_declaration(subprogram_declaration)
        self.assertEqual(subprogram_declaration, file[a_file_range])


if __name__ == '__main__':
    unittest.main()
