import unittest
from file_position import FilePosition, parse


class TestFilePosition(unittest.TestCase):
    def test_invalid_value(self):
        with self.assertRaises(ValueError):
            FilePosition(-1, 1)

    def test_invalid_string_representation(self):
        with self.assertRaises(ValueError):
            parse(":2")

    def test_invalid_string_separator(self):
        with self.assertRaises(ValueError):
            parse("1@2")

    def test_string_representation(self):
        expected = "1:1"
        self.assertEqual(expected, str(parse(expected)))

    def test_eq_different_type(self):
        self.assertFalse(FilePosition(1, 1) == 1)

    def test_lt_true_line(self):
        self.assertTrue(FilePosition(1, 1) < FilePosition(2, 1))

    def test_lt_false_line(self):
        self.assertFalse(FilePosition(2, 1) < FilePosition(1, 1))

    def test_lt_true_same_line(self):
        self.assertTrue(FilePosition(1, 1) < FilePosition(1, 2))

    def test_lt_false_same_line(self):
        self.assertFalse(FilePosition(1, 1) < FilePosition(1, 1))


if __name__ == '__main__':
    unittest.main()
