import unittest
from file_position import FilePosition
from file_range import FileRange, parse


class TestFileRange(unittest.TestCase):
    def test_invalid_range(self):
        first_position = FilePosition(1, 1)
        second_position = FilePosition(2, 2)
        self.assertTrue(first_position < second_position)
        with self.assertRaises(Exception):
            FileRange(second_position, first_position)

    def test_invalid_separator_string_representation(self):
        with self.assertRaises(Exception):
            parse("2:2@1:1")

    def test_invalid_position_string_representation(self):
        with self.assertRaises(Exception):
            parse("2:2-1@1")

    def test_string_representation(self):
        expected = "1:1-2:2"
        self.assertEqual(expected, str(parse(expected)))

    def test_contains_self(self):
        file_range = parse("1:1-2:2")
        self.assertTrue(file_range.contains(file_range))

    def test_not_contains_earlier_start(self):
        first_position = FilePosition(1, 1)
        second_position = FilePosition(2, 2)
        third_position = FilePosition(3, 3)
        self.assertTrue(first_position < second_position)
        self.assertTrue(second_position < third_position)
        first_file_range = FileRange(second_position, third_position)
        second_file_range = FileRange(first_position, third_position)
        self.assertFalse(first_file_range.contains(second_file_range))

    def test_not_contains_later_end(self):
        first_position = FilePosition(1, 1)
        second_position = FilePosition(2, 2)
        third_position = FilePosition(3, 3)
        self.assertTrue(first_position < second_position)
        self.assertTrue(second_position < third_position)
        first_file_range = FileRange(first_position, second_position)
        second_file_range = FileRange(first_position, third_position)
        self.assertFalse(first_file_range.contains(second_file_range))


if __name__ == '__main__':
    unittest.main()
