import unittest

from expression import Expression
from file_path import FilePath
from file_range import parse
from name import Name
from parameter_declaration import ParameterDeclaration
from reference import Reference

a_name: Name = Name("a name")
a_file_path = FilePath("a file path")
a_file_range = parse("1:1-2:2")
a_type_name: Name = Name("a type")


class TestParameterDeclaration(unittest.TestCase):
    parameter_declaration: ParameterDeclaration

    def setUp(self) -> None:
        self.parameter_declaration = ParameterDeclaration(a_name,
                                                          a_file_path,
                                                          a_file_range,
                                                          a_type_name)

    def test_equality(self):
        parameter_declaration_with_same_values = ParameterDeclaration(a_name,
                                                                      a_file_path,
                                                                      a_file_range,
                                                                      a_type_name)
        self.assertEqual(self.parameter_declaration, parameter_declaration_with_same_values)
        self.assertEqual(parameter_declaration_with_same_values, self.parameter_declaration)
        self.assertTrue(parameter_declaration_with_same_values == self.parameter_declaration)
        self.assertTrue(self.parameter_declaration == parameter_declaration_with_same_values)

    def test_inequality(self):
        self.assertNotEqual(self.parameter_declaration, "an other object")
        self.assertNotEqual("an other object", self.parameter_declaration)
        self.assertFalse(self.parameter_declaration == "an other object")
        self.assertFalse("an other object" == self.parameter_declaration)

    def test_hash(self):
        parameter_declaration_with_same_values = ParameterDeclaration(a_name,
                                                                      a_file_path,
                                                                      a_file_range,
                                                                      a_type_name)
        self.assertEqual(hash(self.parameter_declaration),
                         hash(parameter_declaration_with_same_values))

    def test_access(self):
        self.assertEqual(a_name, self.parameter_declaration.name)
        self.assertEqual(a_file_path, self.parameter_declaration.file_path)
        self.assertEqual(a_file_range, self.parameter_declaration.file_range)
        self.assertEqual(a_type_name, self.parameter_declaration.type_name)

    def test_immutable_name(self):
        with self.assertRaises(AttributeError):
            self.parameter_declaration.name = Name("a new name")

    def test_immutable_file_path(self):
        with self.assertRaises(AttributeError):
            self.parameter_declaration.file_path = FilePath("another path")


if __name__ == '__main__':
    unittest.main()
