import unittest
from file_path import FilePath
from file_range import FileRange, parse
from parameter_declaration import ParameterDeclaration
from name import Name
from parameter_declarations import ParameterDeclarations


class TestParameterDeclarations(unittest.TestCase):
    def test_retrieve_parameter_by_position(self):
        parameter_declarations = ParameterDeclarations()
        for i in range(10):
            with self.subTest(str(i)):
                a_name = Name(str(i))
                a_file_path: FilePath = FilePath("a file path")
                a_file_range: FileRange = parse("1:1-2:2")
                a_type = Name("a type")
                a_parameter_declaration = ParameterDeclaration(a_name, a_file_path, a_file_range, a_type)
                parameter_declarations.add(a_parameter_declaration)
                self.assertEqual(i+1, len(parameter_declarations))
                self.assertEqual(a_parameter_declaration, parameter_declarations.get_by_position(i))

    def test_retrieve_parameter_by_name(self):
        parameter_declarations = ParameterDeclarations()
        for i in range(1, 11):
            with self.subTest(str(i)):
                a_name = Name(str(i))
                a_file_path: FilePath = FilePath("a file path")
                a_file_range: FileRange = parse("1:1-2:2")
                a_type = Name("a type")
                a_parameter_declaration = ParameterDeclaration(a_name, a_file_path, a_file_range, a_type)
                parameter_declarations.add(a_parameter_declaration)
                self.assertEqual(i, len(parameter_declarations))
                self.assertEqual(a_parameter_declaration, parameter_declarations.get_by_name(a_name))

    def test_exception_on_add_parameter_with_same_name(self):
        a_name: Name = Name("a name")
        a_file_path: FilePath = FilePath("a file path")
        a_file_range: FileRange = parse("1:1-2:2")
        a_type = Name("a type")
        a_parameter = ParameterDeclaration(a_name, a_file_path, a_file_range, a_type)

        another_file_path: FilePath = FilePath("another file path")
        another_file_range: FileRange = parse("3:3-4:4")
        another_type = Name("another type")
        another_parameter_with_same_name = ParameterDeclaration(a_name,
                                                                another_file_path,
                                                                another_file_range,
                                                                another_type)
        parameters = ParameterDeclarations()
        parameters.add(a_parameter)
        with self.assertRaises(Exception):
            parameters.add(another_parameter_with_same_name)


if __name__ == '__main__':
    unittest.main()
