import unittest
from expression import Expression
from file_path import FilePath
from file_range import parse
from literal import Literal
from name import Name
from tuple_expression import TupleExpression
from parameter import Parameter


class TestTuple(unittest.TestCase):
    def test_tuple_is_expression(self):
        tuple_expression = TupleExpression(tuple())
        self.assertIsInstance(tuple_expression, Expression)

    def test_empty_tuple_is_closed(self):
        tuple_expression = TupleExpression(tuple())
        self.assertTrue(tuple_expression.is_closed())

    def test_closed_only(self):
        list_of_closed_expressions = list()
        for i in range(1, 11):
            with self.subTest(str(i)):
                closed_expression = TupleExpression(tuple())
                assert closed_expression.is_closed(), "Expectation of closed expression is violated"
                list_of_closed_expressions.append(closed_expression)
                tuple_expression = TupleExpression(list_of_closed_expressions)
                self.assertEqual(i, len(tuple_expression))
                self.assertTrue(tuple_expression.is_closed())

    def test_literals_only(self):
        list_of_literal_expressions = list()
        for i in range(1, 11):
            with self.subTest(str(i)):
                literal_expression = Literal("don't care")
                assert isinstance(literal_expression, Literal), \
                    "Expectation of literal expression is violated"
                list_of_literal_expressions.append(literal_expression)
                tuple_expression = TupleExpression(list_of_literal_expressions)
                self.assertEqual(i, len(tuple_expression))
                self.assertTrue(tuple_expression.is_literal())

    def test_non_literal(self):
        name: Name = Name("don't care")
        file_path = FilePath("a file path")
        file_range = parse("1:1-2:2")
        non_literal_expression = Parameter(name, file_path, file_range)
        tuple_expression = TupleExpression([non_literal_expression])
        self.assertFalse(tuple_expression.is_literal())

    def test_open(self):
        name: Name = Name("don't care")
        file_path = FilePath("a file path")
        file_range = parse("1:1-2:2")
        open_expression = Parameter(name, file_path, file_range)
        assert not open_expression.is_closed(), "Expectation of open expression is violated"
        tuple_expression = TupleExpression([open_expression])
        self.assertFalse(tuple_expression.is_closed())

    def test_equality(self):
        """
        Equality by value
        """
        list_of_expressions = list()
        for i in range(1, 11):
            with self.subTest(str(i)):
                literal_expression = Literal(str(i))
                list_of_expressions.append(literal_expression)
                tuple_expression = TupleExpression(list_of_expressions)
                tuple_expression_same_value = TupleExpression(list_of_expressions)
                self.assertEqual(tuple_expression_same_value, tuple_expression)


if __name__ == '__main__':
    unittest.main()
