import unittest

from xml_generation import xml_node


class TestXmlGeneration(unittest.TestCase):
    def test_only_tag(self):
        tag = "tag"
        self.assertEqual(f'<{tag}/>', xml_node(tag))

    def test_tag_and_attributes(self):
        tag = "tag"
        attribute1 = "attribute1"
        value1 = "value1"
        attribute2 = "attribute2"
        value2 = "value2"

        attributes = dict()
        attributes[attribute1] = value1
        attributes[attribute2] = value2
        self.assertEqual(f'<{tag} {attribute1}="{value1}" {attribute2}="{value2}"/>',
                         xml_node(tag, attributes=attributes))

    def test_node_tag_no_newline(self):
        tag = "tag"
        xml = "xml"
        self.assertEqual(f'<{tag}>{xml}</{tag}>', xml_node(tag, xml))

    def test_node_tag_with_newline(self):
        tag = 'tag'
        line1 = 'line1'
        line2 = 'line2'
        indentation = '   '
        self.assertEqual(f'<{tag}>\n{indentation}{line1}\n{indentation}{line2}\n</{tag}>',
                         xml_node(tag, '\n'.join([line1, line2])))

    def test_tag_attributes_and_xml(self):
        tag = "tag"
        attribute1 = "attribute1"
        value1 = "value1"
        attribute2 = "attribute2"
        value2 = "value2"
        xml = "xml"

        attributes = dict()
        attributes[attribute1] = value1
        attributes[attribute2] = value2
        self.assertEqual(f'<{tag} {attribute1}="{value1}" {attribute2}="{value2}">{xml}</{tag}>',
                         xml_node(tag, xml=xml, attributes=attributes))


if __name__ == '__main__':
    unittest.main()
